@@ -64,7 +64,7 @@ struct _MetaWindow
 {
   MetaDisplay *display;
   MetaScreen *screen;
-  GList *workspaces;
+  MetaWorkspace *workspace;
   Window xwindow;
   /* may be NULL! not all windows get decorated */
   MetaFrame *frame;
@@ -207,6 +207,9 @@ struct _MetaWindow
   /* Have we placed this window? */
   guint placed : 1;
 
+  /* Has this window not ever been shown yet? */
+  guint showing_for_first_time : 1;
+
   /* Are we in meta_window_free()? */
   guint unmanaging : 1;
 
@@ -385,6 +388,9 @@ void        meta_window_resize_with_gravity (MetaWindow  *window,
 void        meta_window_fill_horizontal     (MetaWindow  *window);
 void        meta_window_fill_vertical       (MetaWindow  *window);
 
+/* Return whether the window should be currently mapped */
+gboolean    meta_window_should_be_showing   (MetaWindow  *window);
+
 /* This recalcs the window/frame size, and recalcs the frame
  * size/contents as well.
  */
@@ -455,9 +461,6 @@ void meta_window_show_menu (MetaWindow *window,
                             int         button,
                             Time        timestamp);
 
-gboolean meta_window_shares_some_workspace (MetaWindow *window,
-                                            MetaWindow *with);
-
 void meta_window_set_gravity (MetaWindow *window,
                               int         gravity);
 
@@ -466,7 +469,7 @@ void meta_window_handle_mouse_grab_op_event (MetaWindow *window,
 
 GList* meta_window_get_workspaces (MetaWindow *window);
 
-gboolean meta_window_visible_on_workspace (MetaWindow    *window,
+gboolean meta_window_located_on_workspace (MetaWindow    *window,
                                            MetaWorkspace *workspace);
 
 void meta_window_get_work_area_current_xinerama (MetaWindow    *window,
@@ -484,7 +487,7 @@ gboolean meta_window_same_application (MetaWindow *window,
 #define META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE(w) \
   ((w)->type != META_WINDOW_DOCK && (w)->type != META_WINDOW_DESKTOP)
 #define META_WINDOW_IN_NORMAL_TAB_CHAIN(w) \
-  (((w)->input || (w)->take_focus) && META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w))
+  (((w)->input || (w)->take_focus ) && META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w) && (!(w)->skip_taskbar))
 #define META_WINDOW_IN_DOCK_TAB_CHAIN(w) \
   (((w)->input || (w)->take_focus) && ! META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w))
 
